package dev.boyfailure.quajra.vectorbreakout;

import java.util.Arrays;

public class Ball {
    
    short ballX = 250;
    short ballY = 300;
    short ballWidth = 7;
    short ballHeight = 7;
    private boolean isActive = false;
    private boolean isInvincible = false;
    byte ballColor = 0;
    Hitbox hitBox;
    
    byte speedX = 5;
    byte speedY = 5;
    
    private GameState gameState;
    
    public Ball(GameState gameState, int x, int y, int width, int height, int color, boolean active) {
        this.gameState = gameState;
        this.ballX = (short) x;
        this.ballY = (short) y;
        this.ballWidth = (short) width;
        this.ballHeight = (short) height;
        this.ballColor = (byte) color;
        this.hitBox = new Hitbox(x, y, width, height, 1);
        this.isActive = active;
    }
    
    public void spawnBall(boolean useChancey) {
        if (useChancey) {
            this.ballX += (short) (gameState.getChancey() * 3);
            this.ballY += (short) gameState.getChancey();
        }
        gameState.ballsOnScreen++;
        this.isActive = true;
        gameState.lives--;
        gameState.textElementList.get(4).setText(String.valueOf(gameState.lives));
    }
    
    public void spawnBall(int x, int y) {
        this.ballX = (short) x;
        this.ballY = (short) y;
        gameState.ballsOnScreen++;
        this.isActive = true;
        gameState.lives--;
        gameState.textElementList.get(4).setText(String.valueOf(gameState.lives));
    }
    
    public void moveBall() {
        this.ballX += this.speedX;
        this.ballY += this.speedY;
        
        this.hitBox.moveTo(this.ballX, this.ballY, this.ballWidth, this.ballHeight);
        
        this.checkCollision();
    }
    
    public void bounce(int bounceType) {
        switch((byte) bounceType) {
            case 0:
                this.speedY = (byte) -(this.speedY);
                switch (gameState.getChancey()) {
                    case 0: break;
                    case 1: this.speedX++; break;
                    case 2: break;
                    case 3: this.speedX--; break;
                    case 4: break;
                    case 5: this.speedY++;break;
                    case 6: break;
                    case 7: this.speedY--;break;
                    case 8: break;
                    case 9: this.speedY++;speedX++;break;
                    case 10: break;
                    case 11: this.speedY--;speedX--;break;
                    case 12: break;
                    case 13: this.speedY++;speedY++;break;
                    case 14: break;
                    case 15: this.speedY--;speedY--;break;
                }
                break;
            case 1:
                this.speedX = (byte) -(this.speedX);
                break;
            case 2:
                if (this.speedX > 0) {
                    this.speedX = (byte) -(this.speedX);
                }
                else {
                    this.speedX -= 4;
                }
                this.speedY = (byte) -(this.speedY);
                break;
            case 3:
                if (this.speedX < 0) {
                    this.speedX = (byte) -(this.speedX);
                }
                else {
                    this.speedX += 4;
                }
                this.speedY = (byte) -(this.speedY);
                break;
        }
        if (this.speedX < 2 && this.speedX >= 0) {this.speedX = 2;}
        else if (this.speedX > -2 && this.speedX < 0) {this.speedX = -2;}
        else if (this.speedX > 11) {this.speedX -= 3;}  // Slow down if too fast
        if (this.speedY < 4 && this.speedY >= 0) {this.speedY = 4;}
        else if (this.speedY > -4 && this.speedY < 0) {this.speedY = -4;}
        else if (this.speedY > 11) {this.speedY -= 3;} // Slow down if too fast
    }
    
    public void destroyBall() {
        this.isActive = false;
        this.ballX = 32767;
        this.ballY = 32767;
        gameState.ballsOnScreen--;
        if (gameState.ballsOnScreen <= 0) {
            gameState.ballsOnScreen = 0;
            if (gameState.lives <= 0) {
                gameState.gameLose();
            }
        }
    }
    
    // Checks to see if the ball is in contact with bricks, paddles, or walls
    public void checkCollision() {

        this.setInvincibilityState(true); // Delay bouncing again for a frame

        // Check floor
        if (this.ballY >= gameState.getInternalResY()) {
            this.destroyBall();
            return;
        }
        // Check ceiling
        else if (this.ballY <= 0) {
            this.bounce(0);
            this.ballY = 0;
            this.wallBounceAnimation();
            return;
        }
        
        // Check walls
        if (this.ballX <= 20) {
            this.bounce(1);
            this.ballX = 22;
            this.wallBounceAnimation();
            return;
        }
        else if ((this.ballX + this.ballWidth) >= (gameState.getInternalResX() - 20 - this.ballWidth)) {
            this.bounce((byte) 1);
            this.ballX = (short) (gameState.getInternalResX() - 25 - this.ballWidth);
            this.wallBounceAnimation();
            return;
        }
        
        // Check bricks
        if (gameState.currentLevel != null) {
            if (gameState.currentLevel.brickList != null) {
                for (Brick brick : gameState.currentLevel.brickList) {
                    if (this.hitBox.collides(brick.hitBox.getBounds())) {
                        brick.breakBrick();
                        this.bounce(0);
                        gameState.brickCollector.add(brick);
                        return;
                    }
                }
            }
        }
        
        // Check paddles
        for (Paddle paddle : gameState.paddleList) {
            if (this.hitBox.collides(paddle.hitBox.getBounds())) {
                if (this.hitBox.collides(paddle.hitBoxLeft.getBounds())) {
                    this.bounce(2);
                }
                else if (this.hitBox.collides(paddle.hitBoxRight.getBounds())) {
                    this.bounce(3);
                }
                else {
                    this.bounce(0);
                }

                this.ballY -= 2;

                // Draw particles
                Particle particle1, particle2, particle3;
                particle1 = new Particle(gameState, 0, this.ballX, this.ballY - 10, 0, -2, 1, 2, 7, 3);
                particle2 = new Particle(gameState, 0, this.ballX - 3, this.ballY - 10, -2, -4, 1, 2, 6, 3);
                particle3 = new Particle(gameState, 0, this.ballX + 10, this.ballY - 10, 2, -4, 1, 2, 8, 3);
                gameState.particleList.add(particle1);
                gameState.particleList.add(particle2);
                gameState.particleList.add(particle3);
                particle1.spawn(5);
                particle2.spawn(5);
                particle3.spawn(5);
                
                return; // Prevents bouncing multiple times
            }
        }
    }
    
    private void wallBounceAnimation() {
        Particle particle1, particle2, particle3, particle4;
        particle1 = new Particle(gameState, 0, this.ballX - 3, this.ballY - 10, -2, -2, 1, 2, 6, 3);
        particle2 = new Particle(gameState, 0, this.ballX + 10, this.ballY - 10, 2, -2, 1, 2, 8, 3);
        particle3 = new Particle(gameState, 0, this.ballX - 3, this.ballY + 3, 2, 2, 1, 2, 2, 3);
        particle4 = new Particle(gameState, 0, this.ballX + 10, this.ballY + 3, -2, 2, 1, 2, 4, 3);
        gameState.particleList.add(particle1);
        gameState.particleList.add(particle2);
        gameState.particleList.add(particle3);
        gameState.particleList.add(particle4);
        particle1.spawn(5);
        particle2.spawn(5);
        particle3.spawn(5);
        particle4.spawn(5);
    }

    public boolean getActiveState() {
        return this.isActive;
    }
    public void setActiveState(boolean activeState) {
        this.isActive = activeState;
    }

    public boolean getInvincibilityState() {
        return this.isInvincible;
    }
    public void setInvincibilityState(boolean invincibilityState) {
        this.isInvincible = invincibilityState;
    }

}
