package dev.boyfailure.quajra.vectorbreakout;

public class Brick {
    
    short brickX;
    short brickY;
    short brickWidth = 40;
    short brickHeight = 17;
    private boolean isBroken = false;
    byte brickColor = 0;
    byte scoreOnBreak = 10;
    byte specialAbility = 0;
    Hitbox hitBox;
    
    private GameState gameState;
    
    /**
     * Bricks.
     * @param gameState The GameState in which the Brick will reside.
     * @param x The x coordinate of the Brick.
     * @param y The y coordinate of the Brick.
     * @param isBroken Determines whether the Brick is broken.
     * @param brickColor The color code of the Brick.
     * @param score The score the Player earns upon breaking the Brick.
     * @param specialAbility A special event that occurs upon breaking the Brick.
     */
    public Brick(GameState gameState, int x, int y, boolean isBroken, int brickColor, int score, int specialAbility) {
        this.gameState = gameState;
        this.brickX = (short) x;
        this.brickY = (short) y;
        this.isBroken = isBroken;
        this.brickColor = (byte) brickColor;
        this.scoreOnBreak = (byte) score;
        this.hitBox = new Hitbox(x, y, this.brickWidth, this.brickHeight, 0);
        this.specialAbility = (byte) specialAbility;
    }
    
    public void breakBrick() {
        if (!isBroken) {
            isBroken = true;
            gameState.score += this.scoreOnBreak + (gameState.level - 1);
            gameState.textElementList.get(3).setText(String.valueOf(gameState.score));
            gameState.bricksOnScreen--;
            switch(this.specialAbility) {
                case 1:
                    gameState.lives++;
                    break;
                case 2:
                    Ball ball = new Ball(gameState, 300, 300, 7, 7, 0, true);
                    ball.spawnBall((this.brickX + (this.brickWidth / 2)), (this.brickY + (this.brickHeight / 2)));
                    break;
                default:
                    break;
            }
        }
        
        // Draw particles
        Particle particle1, particle2, particle3;
        particle1 = new Particle(gameState, 0, this.brickX + 12, this.brickY + 3, 1, 1, this.brickColor, 2, 4, 1);
        particle2 = new Particle(gameState, 0, this.brickX + 20, this.brickY + 13, 1, 1, this.brickColor, 2, 3, 1);
        particle3 = new Particle(gameState, 0, this.brickX + 32, this.brickY + 7, 1, 1, this.brickColor, 2, 2, 1);
        gameState.particleList.add(particle1);
        gameState.particleList.add(particle2);
        gameState.particleList.add(particle3);
        particle1.spawn(5);
        particle2.spawn(5);
        particle3.spawn(5);
        
        this.brickX = 32767;
        this.brickY = 32767;
        this.hitBox.moveTo(32760, 32760, 2, 2);
    }
    
    public boolean getBrokenState() {
        return this.isBroken;
    }
    public void setBrokenState(boolean brokenState) {
        this.isBroken = brokenState;
    }
    
}
