package dev.boyfailure.quajra.vectorbreakout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;

public class GameState {
    
    public String gameName = "Vector Breakout";
    public String gameVersion = "20250511";
    private boolean isGameStarted = false;
    private boolean isPaused = false;
    private boolean debugMenuEnabled = false;
    public boolean isLost = false;
    public boolean movingLeft = false;
    public boolean movingRight = false;
    public boolean confettiMode = false;
    public boolean collectorActive = false;
    public byte lives = 5;
    private byte chancey = 0;
    private final byte maxChanceyValue = 16;
    public byte ballsOnScreen = 0;
    private short targetFrameRate = 60;
    public short actualFrames = 0;
    private short actualFrameRate = 0;
    public byte gameTickRate = 60;
    private short internalResX = 800;
    private short internalResY = 600;
    private short windowResX = 1024;
    private short windowResY = 768;
    public short level = 1;
    public short bricksOnScreen = 0;
    public int score = 0;
    private long frameCounter = 0;

    public ArrayList<Ball> ballList;
    public ArrayList<Particle> particleList;
    public ArrayList<Paddle> paddleList;
    public ArrayList<TextElement> textElementList;

    public ArrayList<Ball> ballCollector;
    public ArrayList<Particle> particleCollector;
    public ArrayList<Paddle> paddleCollector;
    public ArrayList<TextElement> textElementCollector;
    public ArrayList<Brick> brickCollector;

    public Level currentLevel;
    
    // Debug stuffs
    public boolean debugSpawnAllBalls = false;
    public boolean debugUseGiantPaddle = false;
    public short debugStartLevel = 1;
    
    public GameState() {
        this.initComponents();
        this.showTitleScreen();
    }
    
    public KeyListener getKeyListener() {
        return new GameKeyListener();
    }
    
    public ActionListener getGameStateUpdateActionListener() {
        return new GameStateUpdateListener();
    }
    
    public ActionListener getGameTickActionListener() {
        return new GameTickActionListener();
    }

    class GameTickActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            onGameTick();
        }
    }

    class GameKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            switch(e.getKeyCode()) {
                case 27: // Escape
                    toggleDebugMenu();
                    break;
                case 10: // Enter
                    if (GameState.this.getGameStartedState()) {
                        GameState.this.togglePausedState();
                        if (!GameState.this.getPausedState()) {
                            GameState.this.textElementList.get(6).setVisibility(false);
                        }
                        else {
                            GameState.this.textElementList.get(6).setVisibility(true);
                        }
                    }
                    break;
                case 35: // End
                    if (isLost || !GameState.this.getGameStartedState()) {GameState.this.setGameStartedState(true);GameState.this.newGame(true);}
                    break;
                case 32: // Space
                    if (isLost || !GameState.this.getGameStartedState()) {GameState.this.setGameStartedState(true);GameState.this.newGame();}
                    else if (!GameState.this.getPausedState()) {
                        if (ballsOnScreen < 1) {
                            Ball newBall = new Ball(GameState.this, 200, 300, 7, 7, 0, true);
                            GameState.this.ballList.add(newBall);
                            newBall.spawnBall(true);
                        }
                    }
                    break;
                case 37: // Left
                    if (GameState.this.getGameStartedState()) {movingLeft = true;}
                    break;
                case 39: // Right
                    if (GameState.this.getGameStartedState()) {movingRight = true;}
                    break;
            }
        }
        public void keyReleased(KeyEvent e) {
            switch(e.getKeyCode()) {
                case 37: // Left
                    movingLeft = false;
                    break;
                case 39: // Right
                    movingRight = false;
                    break;
            }
        }
        public void keyTyped(KeyEvent e) {}
    }
    
    class GameStateUpdateListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {

            GameState.this.collectorActive = true;
            GameState.this.ballCollector = new ArrayList<>();
            GameState.this.particleCollector = new ArrayList<>();
            GameState.this.paddleCollector = new ArrayList<>();
            GameState.this.textElementCollector = new ArrayList<>();
            GameState.this.brickCollector = new ArrayList<>();

            bricksOnScreen = 0;
            actualFrameRate = actualFrames;
            actualFrames = 0;
            if (GameState.this.currentLevel != null) {
                if (GameState.this.currentLevel.brickList != null) {
                    for (Brick brick : GameState.this.currentLevel.brickList) {
                        if (brick.getBrokenState()) {
                            GameState.this.brickCollector.add(brick);
                            //GameState.this.currentLevel.brickList.remove(brick);
                        }
                        else {
                            bricksOnScreen++;
                        }
                    }
                }
            }

            for (Ball ball : GameState.this.ballList) {
                if (!ball.getActiveState()) {
                    //GameState.this.ballList.remove(ball);
                    GameState.this.ballCollector.add(ball);
                }
            }

            for (Particle particle: GameState.this.particleList) {
                if (!particle.getActiveState()) {
                    //GameState.this.particleList.remove(particle);
                    GameState.this.particleCollector.add(particle);
                }
            }

            if (bricksOnScreen <= 0 && GameState.this.getGameStartedState() && !isLost) {
                level++;
                GameState.this.currentLevel = new Level(GameState.this, level);
                GameState.this.currentLevel.constructLevel();
            }

            GameState.this.ballList.removeAll(GameState.this.ballCollector);
            GameState.this.paddleList.removeAll(GameState.this.paddleCollector);
            GameState.this.particleList.removeAll(GameState.this.particleCollector);
            GameState.this.textElementList.removeAll(GameState.this.textElementCollector);
            if (GameState.this.currentLevel != null) {
                GameState.this.currentLevel.brickList.removeAll(GameState.this.brickCollector);
            }
            GameState.this.collectorActive = false;

        }
    }

    public void incrementFrameCounter() {
        this.frameCounter++;
    }
    public long getFrameCounter() {
        return this.frameCounter;
    }

    public void incrementChancey() {
        this.chancey++;
        if (this.chancey >= this.maxChanceyValue) {
            this.chancey = 0;
        }
    }
    public byte getChancey() {
        return this.chancey;
    }

    public byte getGameTickRate() {
        return this.gameTickRate;
    }

    public short getTargetFrameRate() {
        return this.targetFrameRate;
    }

    public short getActualFrameRate() {
        return this.actualFrameRate;
    }

    public boolean getPausedState() {
        return this.isPaused;
    }
    public void togglePausedState() {
        this.isPaused = !this.isPaused;
    }

    public boolean getDebugMenuState() {
        return this.debugMenuEnabled;
    }
    public void toggleDebugMenu() {
        this.debugMenuEnabled = !this.debugMenuEnabled;
    }

    public boolean getGameStartedState() {
        return this.isGameStarted;
    }
    public void setGameStartedState(boolean startState) {
        this.isGameStarted = startState;
    }

    public short getInternalResX() {
        return this.internalResX;
    }
    public short getInternalResY() {
        return this.internalResY;
    }
    public short getWindowResX() {
        return this.windowResX;
    }
    public short getWindowResY() {
        return this.windowResY;
    }
    public void setWindowResX(short resolution) {
        this.windowResX = resolution;
    }
    public void setWindowResY(short resolution) {
        this.windowResY = resolution;
    }

    public void onGameTick() {
        this.incrementChancey();
        
        if (this.getPausedState()) {return;}
        
        // Move player if keys are held
        for (Paddle paddle : this.paddleList) {
            if (this.movingLeft) {
                paddle.movePaddle(false);
            }
            else if (this.movingRight) {
                paddle.movePaddle(true);
            }
            else {
                paddle.paddleSpeed = 0;
            }
        }
        
        // Ball logic (movement, collision checks)
        try {
            for (Ball ball : this.ballList) {
                if (ball.getActiveState()) {
                    ball.moveBall();
                }
            }
        } catch(java.util.ConcurrentModificationException except) {
            System.err.println("ConcurrentModificationException, onGameTick() Ball iteration\n"
                             + "Resetting ballList ArrayList.");
            // except.printStackTrace();
            this.ballList = new ArrayList<>();
        }
        
        // Particles
        for (Particle particle : this.particleList) {
            if (particle.getActiveState()) {
                particle.update();
            }
        }
    }
    
    public void showTitleScreen() {
        textElementList.get(0).activateTimer();
        textElementList.get(1).activateTimer();
        textElementList.get(2).activateTimer();
    }
    
    public void gameLose() {
        if (this.currentLevel != null) {
            for (Brick brick : this.currentLevel.brickList) {
                brick.setBrokenState(true);
                this.brickCollector.add(brick);
            }
        }
        for (Ball ball : this.ballList) {
            ball.setActiveState(false);
            this.ballCollector.add(ball);
        }
        for (Paddle paddle : this.paddleList) {
            paddle.setActiveState(false);
            this.paddleCollector.add(paddle);
        }
        this.isLost = true;
        this.ballsOnScreen = 0;
        this.textElementList.get(3).setVisibility(false);
        this.textElementList.get(4).setVisibility(false);
        this.textElementList.get(5).setVisibility(false);
        this.textElementList.get(8).setVisibility(true);
        this.textElementList.get(9).setVisibility(true);
        this.getGameStateUpdateActionListener().actionPerformed(new ActionEvent(this, 0, ""));
    }
    
    public void newGame() {
        newGame(false);
    }
    
    public void newGame(boolean debugLevel) {
        lives = 5;
        isLost = false;
        score = 0;
        if (debugLevel) {level = 32767;}
        else {level = 1;}
        frameCounter = 0;
        ballsOnScreen = 0;
        movingLeft = false;
        movingRight = false;
        if (debugUseGiantPaddle) {this.paddleList.add(new Paddle(this, 0, 500, 800, 60, 0));}
        else {this.paddleList.add(new Paddle(this, 350, 500, 100, 15, 0));}
        if (this.debugUseGiantPaddle) {
            this.paddleList.add(new Paddle(this, 0, 500, 800, 60, 0));
        }
        else {
            this.paddleList.add(new Paddle(this, 350, 500, 100, 15, 0));
        }
        this.textElementList.get(0).setVisibility(false);
        this.textElementList.get(1).setVisibility(false);
        this.textElementList.get(2).setVisibility(false);
        this.textElementList.get(3).setVisibility(true);
        this.textElementList.get(4).setVisibility(true);
        this.textElementList.get(5).setVisibility(true);
        textElementList.get(3).setText(String.valueOf(this.score));
        textElementList.get(4).setText(String.valueOf(this.lives));
        textElementList.get(5).setText(String.valueOf(this.level));
        this.textElementList.get(8).setVisibility(false);
        this.textElementList.get(9).setVisibility(false);
        if (debugStartLevel > 1) {level = debugStartLevel;}
        this.currentLevel = new Level(this, level);
        this.currentLevel.constructLevel();
        this.updateStaticTextElements();
    }

    public void initComponents() {
        this.ballList = new ArrayList<>();
        this.particleList = new ArrayList<>();
        this.paddleList = new ArrayList<>();
        this.textElementList = new ArrayList<>();
                                                                                                    // List of indices
        textElementList.add(new TextElement(this, "VECTOR BREAKOUT", 2.5f, 67, 280, 15, 2));        // 0 - Title
        textElementList.add(new TextElement(this, "MMXXV BOYFAILURE.DEV", 0.75f, 259, 315, 15, 1)); // 1 - Copyright
        textElementList.add(new TextElement(this, "PRESS SPACE TO BEGIN", 1, 215, 345, 15, 2));     // 2 - Space
        textElementList.add(new TextElement(this, String.valueOf(score), 1, 20, -13, 1, 2));        // 3 - Score
        textElementList.add(new TextElement(this, String.valueOf(lives), 1, 200, -13, 1, 2));       // 4 - Lives
        textElementList.add(new TextElement(this, String.valueOf(level), 1, 380, -13, 1, 2));       // 5 - Level
        textElementList.add(new TextElement(this, "PAUSED", 5, 142, 310, 1, 4));                    // 6 - Paused
        textElementList.add(new TextElement(this, "Level Name", 1, 24, 480, 15, 2, 3000));          // 7 - Level Name
        textElementList.add(new TextElement(this, "   GAME OVER   ", 2.5f, 67, 280, 10, 2));        // 8 - Lose
        textElementList.add(new TextElement(this, "PRESS SPACE TO RETRY", 1, 215, 315, 10, 1));     // 9 - Retry
    }
    
    public void updateStaticTextElements() {
        textElementList.get(3).setText(String.valueOf(score));
        textElementList.get(4).setText(String.valueOf(lives));
        textElementList.get(5).setText(String.valueOf(level));
    }

}
